@@ -1,5 +1,5 @@
 /* GMM (C) 2018 Stephane Charette <stephanecharette@gmail.com>
- * $Id: TabCanvas.cpp 2630 2018-10-13 06:37:31Z stephane $
+ * $Id: TabCanvas.cpp 2637 2018-10-16 05:55:54Z stephane $
  */
 
 #include "GMM.hpp"
@@ -8,8 +8,8 @@
 TabCanvas::TabCanvas(const std::string & name) :
 	Thread("thread for device prefix " + name),
 	prefix(name),
-	read_len(0),
 	tab_is_active(false),
+	export_image(cfg().get_bool(prefix + "_export_image")),
 	timer_frequency_in_milliseconds(cfg().get_int(prefix + "_chart_redraw_ms")),
 	comm_handle(INVALID_HANDLE_VALUE)
 {
@@ -194,38 +194,6 @@ void TabCanvas::timerCallback(void)
 
 void TabCanvas::periodic_update(void)
 {
-	// randomly assign some new values as if we're reading from the serial port
-#if 0
-	Random r;
-	for (size_t idx=0; idx < number_of_buckets; idx++)
-	{
-		// the closer to middle of the range, the greater chance we have of getting a board
-		double chance = 0.0;
-		switch (idx)
-		{
-			case 0:	case 17:							chance = 0.98;		break;
-			case 1:	case 16:							chance = 0.95;		break;
-			case 2:	case 3:		case 14:	case 15:	chance = 0.85;		break;
-			case 4:	case 5:		case 12:	case 13:	chance = 0.65;		break;
-			case 6:	case 11:	case 7:		case 10:	chance = 0.55;		break;
-			case 8: case 9:								chance = 0.25;		break;
-		}
-		if (r.nextDouble() >= chance)
-		{
-			total_moisture_content[idx] ++;
-			local_moisture_content[idx] ++;
-		}
-	}
-#endif
-
-#if 0
-	// add the local bucket numbers to the totals
-	for (size_t idx = 0; idx < number_of_buckets; idx++)
-	{
-		total_moisture_content[idx] += local_moisture_content[idx];
-	}
-#endif
-
 	calculate_averages_and_std_deviations();
 
 	// re-populate the fields with the amounts stored in the buckets
@@ -235,8 +203,8 @@ void TabCanvas::periodic_update(void)
 		local_distribution[idx].setText(String(local_moisture_content[idx]));
 	}
 
-	// only re-create the chart image if the tab is active
-	if (tab_is_active)
+	// don't bother redrawing the chart unless we have to
+	if (tab_is_active || export_image)
 	{
 		auto rect = chart.getBounds();
 		chart.setImage(draw_chart(rect.getWidth(), rect.getHeight()));
@@ -319,7 +287,7 @@ void TabCanvas::calculate_averages_and_std_deviations()
 
 Image TabCanvas::draw_chart(const size_t width, const size_t height)
 {
-	if (tab_is_active == false)
+	if (tab_is_active == false && export_image == false)
 	{
 		// don't bother creating images for inactive tabs
 		return Image();
@@ -334,7 +302,7 @@ Image TabCanvas::draw_chart(const size_t width, const size_t height)
 	g.setColour(Colours::black);
 	g.drawRect(0, 0, width, height);
 
-	if (width < 100 || height < 100)
+	if (width < 75 || height < 75)
 	{
 		// too small of an image for us to do anything useful
 		return image;
@@ -346,14 +314,22 @@ Image TabCanvas::draw_chart(const size_t width, const size_t height)
 	g.drawText(cfg().get_str(prefix + "_name"), edge_offset, edge_offset, width - edge_offset * 2, edge_offset * 2, Justification::topLeft);
 	g.drawText(Time::getCurrentTime().formatted("%Y-%m-%d %H:%M:%S"), edge_offset, edge_offset, width - edge_offset * 2, edge_offset * 2, Justification::topRight);
 
-	// some numbers are used in many places, so calculate them now so we don't have to do it again
+	// some numbers are used in many places, so calculate them now so we don't have to do it multiple times
 	const float width_float				= static_cast<float>(width);
 	const float height_float			= static_cast<float>(height);
 	const float h_step					= 1.0f / static_cast<float>(number_of_buckets);
 	const float v_step					= 1.0f / 5.0f;
 	const float width_of_columns		= h_step * width_float;
-	const float one_seventh_of_column	= width_of_columns / 7.0f;
-	const float three_seventh_of_column	= 3.0f * one_seventh_of_column;
+	const float one_seventh_of_column	= 1.0f * width_of_columns / 7.0f;
+	const float three_seventh_of_column	= 3.0f * width_of_columns / 7.0f;
+
+	/* For drawing purposes, each column is divided into 7 parts.
+	 * - 1st part is empty space.
+	 * - 2nd through 4th is the pink.
+	 * - 4th through 6th is the blue.
+	 * - 7th is empty space.
+	 * Note the 4th section is where the pink and blue parts overlap.
+	 */
 
 	// horizontal lines
 	g.setColour(Colours::dimgrey);
@@ -450,12 +426,15 @@ Image TabCanvas::draw_chart(const size_t width, const size_t height)
 		}
 	}
 
-	// save to a .png file for debug purposes
-	File file = File::getSpecialLocation(File::SpecialLocationType::tempDirectory).getChildFile(prefix + "_chart.png");
-	file.deleteFile();
-	FileOutputStream stream(file);
-	PNGImageFormat png;
-	png.writeImageToStream(image, stream);
+	if (export_image)
+	{
+		// save to a .png file for debug purposes
+		File file = File::getSpecialLocation(File::SpecialLocationType::tempDirectory).getChildFile(prefix + "_chart.png");
+		file.deleteFile();
+		FileOutputStream stream(file);
+		PNGImageFormat png;
+		png.writeImageToStream(image, stream);
+	}
 
 	return image;
 }
@@ -576,14 +555,26 @@ void TabCanvas::device_thread(void)
 {
 	// This is started on a secondary thread.
 
-	std::string device_name = cfg().get_str(prefix + "_device");
+	const std::string device_name	=	cfg().get_str	(prefix + "_device"				);
+	const std::string tab_name		=	cfg().get_str	(prefix + "_name"				);
+	const int interval_ms			=	cfg().get_int	(prefix + "_interval_ms"		);
+	const int read_len				=	cfg().get_int	(prefix + "_read_len"			);
+	const bool run_simulation		=	cfg().get_bool	(prefix + "_run_simulation"		);
+	const std::regex rx				(	cfg().get_str	(prefix + "_regex_pattern"		));
 
-	LOG_MSG("device thread for " << device_name << " has started (prefix=" << prefix << ")");
+	LOG_MSG("device thread for " << device_name << " has started (name=\"" << tab_name << "\", prefix=\"" << prefix << "\")");
 
-	const int interval_ms = cfg().get_int(prefix + + "_interval_ms");
+	for (const auto k : cfg().getAllProperties().getAllKeys())
+	{
+		if (k.startsWith(prefix))
+		{
+			const std::string key = k.toStdString();
+			LOG_MSG(device_name << ": " << key << "=\"" << cfg().get_str(key) << "\"");
+		}
+	}
 
-	read_len = cfg().get_int(prefix + "_read_len");
-	read_buffer = std::vector<uint8_t>(read_len, '\0');
+	// The buffer into which bytes are read from the comm port.  The length of this buffer is defined by the *_read_len value in configuration.
+	VBytes read_buffer(read_len, '\0');
 	LOG_MSG(device_name << ": created a read buffer capable of reading " << read_len << " bytes every " << interval_ms << " milliseconds");
 
 	// give this thread a priority boost
@@ -591,35 +582,31 @@ void TabCanvas::device_thread(void)
 
 	open_serial_device(device_name);
 
-	const std::regex rx(cfg().get_str(prefix + "_regex_pattern"));
-
-	const bool run_simulation = cfg().get_bool(prefix + "_run_simulation");
-
 	auto interval	= std::chrono::milliseconds(interval_ms);
 	auto time_now	= std::chrono::high_resolution_clock::now();
 	auto next_read	= time_now + interval;
 
 	while (threadShouldExit() == false)
 	{
-		std::string buffer;
+		std::string str;
 		if (run_simulation)
 		{
-			buffer = get_simulated_string();
+			str = get_simulated_string(read_buffer);
 		}
 		else
 		{
-			buffer = read_from_device();
+			str = read_from_device(read_buffer);
 		}
 
-		if (buffer.empty() == false)
+		if (str.empty() == false)
 		{
 			std::smatch matches;
-			const bool found = std::regex_search(buffer, matches, rx);
+			const bool found = std::regex_search(str, matches, rx);
 
 			if (found)
 			{
 				const double data = std::stod(matches[1]);
-				LOG_MSG(device_name << ": str=" << String(buffer).trim().toStdString() << ", data=" << data);
+				LOG_MSG(device_name << ": str=" << String(str).trim().toStdString() << ", data=" << data);
 
 				// determine which bucket this should fit into
 				int idx = static_cast<int>(data) - percentage_of_first_bucket;
@@ -633,12 +620,12 @@ void TabCanvas::device_thread(void)
 			else
 			{
 				std::string msg;
-				for (const char c : buffer)
+				for (const char c : str)
 				{
 					if (c > 32 && c < 127)	msg += c;
 					else					msg += "<" + std::to_string((int)c) + ">";
 				}
-				LOG_MSG(device_name << ": failed to parse string: len=" << buffer.size() << ": \"" << msg << "\"");
+				LOG_MSG(device_name << ": failed to parse string: len=" << str.size() << ": \"" << msg << "\"");
 			}
 		}
 
